// C++ Sample file to test ActiveX Server Object CornerTracker.CoCornerTracker

#define _WIN32_DCOM
#define _WIN32_WINNT 0x0400

#include <stdio.h>
#include <conio.h>


// THIS PROGRAM USES MFC
// PLEASE COMPILE IT FOR WINDOWS WITH MFC SUPPORT


//THIS PROGRAM REQUIRES MICROSOFT VISION SDK. 
//PLEASE MAKE SURE TO HAVE IT INSTALLED ON YOUR MACHINE

#include "VisWin.h"
#include "VisDisplay.h"

//REQUIRED FOR COM EXTENDED FUNCTIONS


//LOAD TYPE INFORMATION DATA
#import "CornerTracker.tlb" no_namespace named_guids

class CMyWinApp : public CWinApp
{
public:
    int Run();
};

CMyWinApp vapp;

BOOL CMyWinApp::Run()
{	
	//INITIALIZE COM
	HRESULT hr = CoInitializeEx(NULL, COINIT_MULTITHREADED);
	if(FAILED(hr))
	{
		AfxMessageBox("Unable to Initialize COM");
		return -1;
	}


	//CREATE ACTIVEX OBJECT
	ICornerTracker *ptr;
	hr = CoCreateInstance(	CLSID_CoCornerTracker, NULL, CLSCTX_INPROC_SERVER, 
							IID_ICornerTracker, (void**)&ptr);	
	if(FAILED(hr))
	{
		AfxMessageBox("Unable to Create Object");
		return -1;
	}

	

	//READING TEST IMAGES
	CVisGrayByteImage gray1, gray2;
	if (!gray1.FReadFile("image1.bmp"))
	{
		AfxMessageBox("Could not open image 1");
		return -1;
	}
	if (!gray2.FReadFile("image2.bmp"))
	{
		AfxMessageBox("Could not open image 2\n");
		return -1;
	}

	
	//CONFIGURE OBJECT COMPATIBLE WITH IMAGE SIZE
	long image_lines = gray1.Height();
	long image_columns = gray1.Width();	
	hr = ptr->put_lines(image_lines);
	hr = ptr->put_columns(image_columns);

	//ALLOCATE THE CORNER TRACKER
	hr = ptr->create_corner_tracker();
	if(FAILED(hr))
	{
		AfxMessageBox("Unable to Allocate Internal Object Resources");
		return -1;
	}


	//ASK FOR THE DIRECT ACCESS INTERFACE
	_ICornerTracker *ptr_direct;
	hr = ptr->QueryInterface(IID__ICornerTracker, (void**)&ptr_direct);
	if(FAILED(hr))
	{
		AfxMessageBox("Unable to obtain direct interface");
		return -1;
	}


	//DEFINE THE POINTS TO TRACK FOR IMAGE1;
	//YOU CAN USE THE CORNERDETECTOR ACTIVEX OBJECT TO DO THIS
	float points_to_track[] = {	150.0,	121.0,
								147.0,	102.0,
								18.0,	197.0,
								219.0,	86.0,
								121.0,	117.0,
								207.0,	88.0};

	
	//ALLOCATE ARRAYS TO STORE MATCHING INFORMATION
	long count = sizeof(points_to_track)/2/sizeof(float);
	float *matches = (float*)malloc(count*2*sizeof(float));
	float *residue = (float*)malloc(count*sizeof(float));
	byte *valid_match = (byte*)malloc(count*sizeof(byte));
	
	//TRACK THE CORNERS
	hr = ptr_direct->_track_corners(	gray1.MemBlock().PbData(), 
										gray2.MemBlock().PbData(),
										count,
										points_to_track,
										matches,
										valid_match,
										residue );
	if(FAILED(hr))
	{
		AfxMessageBox("Tracking function failed");
		return -1;
	}

	
	// CONVERT GRAY IMAGES TO COLOR IMAGE FOR VISUALIZATION PURPOSES	
	CVisRGBAByteImage color1, color2;
	color1.Allocate(gray1.Rect());
	gray1.CopyPixelsTo(color1);
	color2.Allocate(gray1.Rect());
	gray2.CopyPixelsTo(color2);

	//PLOT CORNERS ON THE IMAGES
	HDC hdcImage;	
	hdcImage = color1.Hdc();
	HPEN pen = ::CreatePen(PS_SOLID, 2, RGB(255,0,0));
	if (hdcImage != 0)
	{
		::SelectObject(hdcImage, pen );
		float x,y;
		for(int i = 0; i < count; i++ )
		{
			x = points_to_track[2*i];
			y = points_to_track[2*i+1];
			::Ellipse(hdcImage, x-2, y-2, x+2, y+2);
		}
		color1.DestroyHdc();
	}
	else
		AfxMessageBox("Could not create device context for test image");

	hdcImage = color2.Hdc();
	if (hdcImage != 0)
	{
		::SelectObject(hdcImage, pen );
		float x,y;
		for(int i = 0; i < count; i++ )
		{
			if(valid_match[i])
			{
				x = matches[2*i];
				y = matches[2*i+1];
				::Ellipse(hdcImage, x-2, y-2, x+2, y+2);
			}
		}
		color2.DestroyHdc();
	}
	else
		AfxMessageBox("Could not create device context for test image");
	::DeleteObject(pen);

	AfxMessageBox("Displaying tracked corners on images");
	CVisPane pane = VisDisplayImage(color1);	
	Sleep(500);
	pane.Display(color2,evispanedispDefault,true);
	Sleep(500);
	pane.Display(color1,evispanedispDefault,true);
	Sleep(500);
	pane.Display(color2,evispanedispDefault,true);
	Sleep(500);
	pane.Display(color1,evispanedispDefault,true);
	Sleep(500);
	pane.Display(color2,evispanedispDefault,true);
	Sleep(500);
	pane.Display(color1,evispanedispDefault,true);
	Sleep(500);
	pane.Display(color2,evispanedispDefault,true);
	Sleep(500);
	pane.Display(color1,evispanedispDefault,true);
	Sleep(2000);
	
	AfxMessageBox("Done");
	//CLOSE EVERYTHING
	ptr->destroy_corner_tracker();
	ptr_direct->Release();
	ptr->Release();
	free(matches);
	free(residue);
	free(valid_match);

	CoUninitialize();
	pane.DestroyWnd();
	return 0;
}

